#include <iostream>
#include "systemc.h"

// header file: process_1.h
SC_MODULE( process_1 ) {
  // Ports
  sc_in_clk clk;
  sc_in<int> a;
  sc_in<bool> ready_a;
  sc_out<int> b;
  sc_out<bool> ready_b;
  // Process functionality
  void do_process_1();
  // Constructor
  SC_CTOR( process_1 ) {
    SC_CTHREAD(do_process_1,clk.pos());
  }
};

// header file: process_2.h
SC_MODULE( process_2 ) {
  // Ports
  sc_in_clk clk;
  sc_in<int> a;
  sc_in<bool> ready_a;
  sc_out<int> b;
  sc_out<bool> ready_b;
  // Process functionality
  void do_process_2();
  // Constructor
  SC_CTOR( process_2 ) {
    SC_CTHREAD( do_process_2 , clk.pos() );
  }
};

// implementation file: process_1.cc
void process_1::do_process_1() {
  int v;
  while ( true ) {
    do wait(); while ( ready_a.read() != true ); 
    v = a.read();
    v += 5;
    cout << "P1: v = " << v << endl;
    b.write( v );
    ready_b.write( true );
    wait();
    ready_b.write( false );
  }
}

// implementation file: process_2.cc
void process_2::do_process_2()  {
  int v;
  while ( true ) {
    do wait(); while ( ready_a.read() != true ); 
    v = a.read();
    v += 3;
    cout << "P2: v = " << v << endl;
    b.write( v );
    ready_b.write( true );
    wait();
    ready_b.write( false );
  }
}

// implementation file: main.cc
int sc_main (int ac,char *av[]) {
  //sc_report_handler::set_actions("/IEEE_Std_1666/deprecated", SC_DO_NOTHING);
  sc_signal<int> s1 ( "Signal-1" );
  sc_signal<int> s2 ( "Signal-2" );
  sc_signal<bool> ready_s1 ( "Ready-1" );
  sc_signal<bool> ready_s2 ( "Ready-2" );
  sc_clock clock( "Clock" , 20 , 0.5 , 0.0 );
  process_1 p1 ( "P1" );
  p1.clk( clock );
  p1.a( s1 );
  p1.ready_a( ready_s1 );
  p1.b( s2 );
  p1.ready_b( ready_s2 );
  process_2 p2 ( "P2" );
  p2.clk( clock );
  p2.a( s2 );
  p2.ready_a( ready_s2 );
  p2.b( s1 );
  p2.ready_b( ready_s1 );
  s1.write(0);
  s2.write(0);
  ready_s1.write(true);
  ready_s2.write(false);
  sc_start(100000, SC_NS);
  return 0;
}
